package hardcaml_axi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Make.Datapath_registerSource

When placed between two components which produce/consume an AXI stream, this module ensures that every output signal is registerd. It fully supports the tvalid/tready handshake protocol.

Sourcemodule IO : sig ... end
Sourcemodule I : sig ... end
Sourcemodule Pipeline_stage_descr : sig ... end

Instantiates a chain of n Datapath_register components and wire up the source and dest signals appropriately.

In most cases, you probably want to use pipeline_simple.

Sourceval pipeline_simple : ?instance_name:Base.string -> n:Base.int -> Hardcaml.Scope.t -> Hardcaml.Signal.t I.t -> Hardcaml.Signal.t IO.t

Construacts a datapath register pipeline with n stages, where all the pipeline stages have the same clear and same instance name.